"""Move PositiveRestricted/NegativeRestricted nodes below ReferenceGrad nodes"""

from ufl.algorithms import MultiFunction
from ufl.classes import ReferenceGrad, PositiveRestricted, NegativeRestricted

from dune.perftool.ufl.transformations import ufl_transformation


class OrderGradientRestriction(MultiFunction):
    call = MultiFunction.__call__

    def __call__(self, o):
        return self.call(o)

    def expr(self, o):
        return self.reuse_if_untouched(o, *tuple(self(op) for op in o.ufl_operands))

    def positive_restricted(self, o):
        if isinstance(o.ufl_operands[0], ReferenceGrad):
            return ReferenceGrad(PositiveRestricted(o.ufl_operands[0].ufl_operands[0]))
        return self.reuse_if_untouched(o, *tuple(self(op) for op in o.ufl_operands))

    def negative_restricted(self, o):
        if isinstance(o.ufl_operands[0], ReferenceGrad):
            return ReferenceGrad(NegativeRestricted(o.ufl_operands[0].ufl_operands[0]))
        return self.reuse_if_untouched(o, *tuple(self(op) for op in o.ufl_operands))


@ufl_transformation(name="order_gradient_restriction")
def order_gradient_restriction(e):
    """Move PositiveRestricted/NegativeRestricted nodes below ReferenceGrad nodes"""
    return OrderGradientRestriction()(e)
